''' <summary>
''' Title und/oder Task blinken lassen
''' </summary>
''' <remarks></remarks>
Public Class FlashWindow

    Private Declare Auto Function FlashWindowEx Lib "user32" _
        (ByRef FWI As FLASHWINFO) As Boolean

    Public Enum FlashWindowMode
        FlashTitle = 1
        FlashTask = 2
        FlashAll = FlashTitle Or FlashTask
    End Enum

    Private Structure FLASHWINFO
        Dim cbSize As Integer
        Dim hwnd As IntPtr
        Dim dwFlags As Integer
        Dim uCount As Integer
        Dim dwTimeout As Integer
    End Structure

    Private mNI As NotifyIcon = Nothing
    Private NI1 As Icon = Nothing
    Private NI2 As Icon = Nothing
    Private mFlash As Boolean = False
    Private WithEvents T As New Timer
    Private ShowIt As Boolean = False

    Public Event FlashStarted()
    Public Event FlashStopped()

    ''' <summary>
    ''' Init FlashWindow
    ''' </summary>
    ''' <param name="NI">
    '''  Optional kann ein NotifyIcon Control bergeben werden
    ''' </param>
    ''' <param name="mFlashNotifyIcon">
    '''  Soll bei StartFlash das Icon blinken?
    ''' </param>
    ''' <remarks></remarks>
    Public Sub New(Optional ByVal NI As NotifyIcon = Nothing, _
        Optional ByVal mFlashNotifyIcon As Boolean = False)

        NotifyIcon = NI
        mFlash = mFlashNotifyIcon
    End Sub

    ''' <summary>
    ''' Ruft ab oder legt das NotifyIcon der Parentform fest 
    ''' </summary>
    Public Property NotifyIcon() As NotifyIcon
        Get
            Return mNI
        End Get
        Set(ByVal value As NotifyIcon)
            mNI = value
            If mNI Is Nothing Then
                NI1 = Nothing
                NI2 = Nothing
                mFlash = False
            Else
                NI1 = mNI.Icon
                NI2 = Icon.FromHandle(New Bitmap(16, 16).GetHicon)
            End If
        End Set
    End Property

    ''' <summary>
    ''' Ruft ab oder legt fest, ob bei StartFlash das Icon blinken soll
    ''' </summary>
    Public Property FlashNotifyIcon() As Boolean
        Get
            Return mFlash
        End Get
        Set(ByVal value As Boolean)
            mFlash = value
        End Set
    End Property

    ''' <summary>
    ''' Blinkvorgang starten
    ''' </summary>
    ''' <param name="Frm">
    '''  Form deren Titelleiste und/oder Task blinken soll
    ''' </param>
    ''' <param name="FlashMode">Title, Task oder beide</param>
    ''' <param name="FlashCount">Anzahl Blinkvorgnge </param>
    ''' <param name="Intervall">Abstand in ms zwischen Blinkumschaltung</param>
    Public Sub StartFlash(ByVal Frm As Form, _
        ByVal FlashMode As FlashWindowMode, _
        Optional ByVal FlashCount As Integer = 100000, _
        Optional ByVal Intervall As Integer = 500)

        Flash(Frm, FlashMode, FlashCount, Intervall)

        RaiseEvent FlashStarted()
        If mFlash Then
            ShowIt = True
            T.Interval = Intervall
            T.Start()
        End If
    End Sub

    ''' <summary>
    ''' Blinkvorgang anhalten
    ''' </summary>
    Public Sub StopFlash(ByVal Frm As Form)
        Flash(Frm, 0, 0, 0)

        T.Stop()
        If mNI IsNot Nothing Then
            mNI.Icon = NI1
        End If
        RaiseEvent FlashStopped()
    End Sub


    Private Sub Flash(ByVal Frm As Form, _
        ByVal FlashMode As FlashWindowMode, _
        Optional ByVal FlashCount As Integer = 100000, _
        Optional ByVal Intervall As Integer = 500)

        Dim FlashInfo As New FLASHWINFO
        FlashInfo.cbSize = _
            System.Runtime.InteropServices.Marshal.SizeOf(FlashInfo)
        FlashInfo.dwFlags = FlashMode
        FlashInfo.dwTimeout = Intervall
        FlashInfo.hwnd = Frm.Handle
        FlashInfo.uCount = FlashCount
        FlashWindowEx(FlashInfo)
    End Sub

    ''' <summary>
    ''' Steuerung Blinkvorgang des Icons vom NotifyIcon
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub T_Tick(ByVal sender As Object, _
        ByVal e As System.EventArgs) Handles T.Tick

        If ShowIt Then
            mNI.Icon = NI1
        Else
            mNI.Icon = NI2
        End If
        ShowIt = Not ShowIt
    End Sub
End Class
